open Mlpost
open Metapost
open Command
open Picture
open Path
open Num
open Num.Infix
open Helpers
open Box
open Color

let pen = Pen.transform [Transform.scaled (bp 3.)] Pen.default

let tabular l =
  "\\begin{tabular}{c}" ^
    String.concat "\\\\\n" l ^
    "\\end{tabular}"

let rec listn n x = if n = 0 then [] else x :: listn (n-1) x

let rose = rgb8 255 204 184
let wbox = round_rect ~stroke:(Some orange) ~fill:lightyellow ~dy:zero
let obox = round_rect ~stroke:None ~fill:rose ~dy:zero
let tbox = round_rect ~stroke:None ~fill:lightblue ~dy:zero

let overview =
  let twhyml = tex "\\bf ~~~~~~~~~~~~~~WhyML" in
  let twhy = tex "\\bf ~~~~~~~~~~~~~~~~~~Why" in
  let vcgen = obox (tex "VCgen") in
  let whyml =
    vbox ~pos:`Center ~padding:(bp 5.)
      [twhyml;
       vcgen;
       empty ()]
  in
  let trans = obox (tex "transform/translate") in
  let run = obox (tex "print/run") in
  let why =
    vbox ~pos:`Center ~padding:(bp 10.)
      [twhy;
       trans;
       run;
       empty ()
      ]
  in
  let mlw_file = tbox (tex "file.mlw") in
  let why_file = tbox (tex "file.why") in
  let coq = tbox (tex "Coq") in
  let alt_ergo = tbox (tex "Alt-Ergo") in
  let gappa = tbox (tex "Gappa") in
  let z3 = tbox (tex "Z3") in
  let etc = tex ~dx:zero "\\!\\!\\!etc." in
  let b =
    vbox ~padding:(bp 15.)
      [vbox ~padding:(bp 15.)
         [hbox ~padding:(bp 20.) ~pos:`Bot
            [why_file; vbox  ~padding:(bp 15.) [mlw_file; wbox whyml]
            ];
	  wbox why];
       hbox ~padding:(bp 10.) [coq; alt_ergo; gappa; z3; etc];
      ]
  in
  let head = Arrow.head_classic ~color:red ~pen in
  let kind =
    Arrow.add_line ~color:red  ~pen (Arrow.add_head ~head Arrow.empty)
  in
  let arrow ?ind ?outd x y =
    Arrow.box_to_box ~kind ?ind ?outd (sub x b) (sub y b) ~sep:(bp 5.)
  in
  draw b ++ arrow vcgen trans ++ arrow trans run ++
  arrow mlw_file vcgen ++ arrow why_file trans ++
  arrow run coq ++ arrow run alt_ergo ++ arrow run gappa ++ arrow run z3

let () = Metapost.emit "overview" overview

open Tree

let bin s = bin ~arrow_style:Undirected ~cs:(bp 20.) (tex s)
let empty = leaf (empty ())
let leaf s = bin s empty empty

let same_fringe =
  let t1 = bin "8" (bin "3" (leaf "1") (bin "5" (leaf "4") empty)) empty in
  let t2 = bin "4" (bin "1" empty (leaf "3")) (bin "8" (leaf "5") empty) in
  hbox ~pos:`Top ~padding:(bp 40.) [to_box t1; to_box t2]

let () = Metapost.emit "same_fringe" (Box.draw same_fringe)

(*
Local Variables:
compile-command: "mlpost -native -latex why3.tex -xpdf figures2.ml"
End:
*)

